/**************************************************************************//**
 * @file     main.c
 * @version  V1.00
 * @brief    An example porting UCOSii_V2.93 on M460 series MCU together with 4CAN transmit-receive.
 * SPDX-License-Identifier: Apache-2.0
 * @copyright (C) 2023 Nuvoton Technology Corp. All rights reserved.
 *
 ******************************************************************************/
#include "stdio.h"
#include "NuMicro.h"
#include "ucos_ii.h"
#include "CANFD_config.h"

#define  __M467SJHAE

/*------------------------------------------------------------------------------------------------*/
/* Define                                                                                         */
/*------------------------------------------------------------------------------------------------*/
#define PLL_CLOCK   200000000
#define RXBUFSIZE   1024

#define LED_R_0_LIGHT()                 (PH4 = 0)
#define LED_R_1_DARK()                  (PH4 = 1)
#define LED_R_TOGGLE()                  (PH4 ^= 1)
#define LED_Y_TOGGLE()                  (PH5 ^= 1)
#define LED_G_TOGGLE()                  (PH6 ^= 1)

#define OS_HIGHEST_PRIO            4
#define TASKA_PRIO                 (OS_LOWEST_PRIO - 10u)
#define TASK0_PRIO                 (OS_LOWEST_PRIO - 9u)
#define TASK1_PRIO                 (OS_LOWEST_PRIO - 8u)
#define TASK2_PRIO                 (OS_LOWEST_PRIO - 7u)
#define TASK3_PRIO                 (OS_LOWEST_PRIO - 6u)

#define TASK_STK_SIZE                    2048

/*------------------------------------------------------------------------------------------------*/
/* Global variables                                                                               */
/*------------------------------------------------------------------------------------------------*/

void *Print_Message[10] ;                                     // Can accommodate 10 messages
OS_EVENT *pQ_print;

int32_t Mail_Message[64] ;                                    // a string
OS_EVENT *p_MailBox_1;

OS_EVENT *pMutex_CANFD0_Tx;
OS_EVENT *pMutex_CANFD1_Tx;
OS_EVENT *pMutex_CANFD2_Tx;
OS_EVENT *pMutex_CANFD3_Tx;

OS_EVENT *pSem_CANFD0_Rx;                            // used it in interrupt subroutines
OS_EVENT *pSem_CANFD1_Rx;
OS_EVENT *pSem_CANFD2_Rx;
OS_EVENT *pSem_CANFD3_Rx;

CANFD_FD_MSG_T      g_sRxMsgFrame;

OS_STK task_High[TASK_STK_SIZE];
OS_STK taskA_stk[TASK_STK_SIZE];
OS_STK task0_stk[TASK_STK_SIZE];
OS_STK task1_stk[TASK_STK_SIZE];
OS_STK task2_stk[TASK_STK_SIZE];
OS_STK task3_stk[TASK_STK_SIZE];

void task_Highest(void *pvParameters);
void task_A(void *pvParameters);
void task0(void *pvParameters);
void task1(void *pvParameters);
void task2(void *pvParameters);
void task3(void *pvParameters);

/*------------------------------------------------------------------------------------------------*/
/* Define functions prototype                                                                     */
/*------------------------------------------------------------------------------------------------*/

void SYS_Init(void)
{
    CLK->PCLKDIV = CLK_PCLKDIV_APB0DIV_DIV2 | CLK_PCLKDIV_APB1DIV_DIV2 ; // Set PCLK0&PCLK1 =HCLK/2
    CLK_SetCoreClock(200000000);                                         // Set HCLK =200MHz

    CLK->AHBCLK0 |= CLK_AHBCLK0_GPACKEN_Msk | CLK_AHBCLK0_GPBCKEN_Msk    // Enable all GPIO clock
                    | CLK_AHBCLK0_GPCCKEN_Msk | CLK_AHBCLK0_GPDCKEN_Msk | CLK_AHBCLK0_GPECKEN_Msk
                    | CLK_AHBCLK0_GPFCKEN_Msk | CLK_AHBCLK0_GPGCKEN_Msk | CLK_AHBCLK0_GPHCKEN_Msk;
    CLK->AHBCLK1 |= CLK_AHBCLK1_GPICKEN_Msk | CLK_AHBCLK1_GPJCKEN_Msk;

    CLK_SetModuleClock(UART0_MODULE, CLK_CLKSEL1_UART0SEL_HIRC, CLK_CLKDIV0_UART0(1));
    CLK_EnableModuleClock(UART0_MODULE);                                       // Enable UART0 clock

    CLK_SetModuleClock(CANFD0_MODULE, CLK_CLKSEL0_CANFD0SEL_HCLK, CLK_CLKDIV5_CANFD0(1));
    CLK_SetModuleClock(CANFD1_MODULE, CLK_CLKSEL0_CANFD1SEL_HCLK, CLK_CLKDIV5_CANFD1(1));
    CLK_SetModuleClock(CANFD2_MODULE, CLK_CLKSEL0_CANFD2SEL_HCLK, CLK_CLKDIV5_CANFD2(1));
    CLK_SetModuleClock(CANFD3_MODULE, CLK_CLKSEL0_CANFD3SEL_HCLK, CLK_CLKDIV5_CANFD3(1));
    CLK_EnableModuleClock(CANFD0_MODULE);                                    // Enable CAN FD0 clock
    CLK_EnableModuleClock(CANFD1_MODULE);                                    // Enable CAN FD1 clock
    CLK_EnableModuleClock(CANFD2_MODULE);                                    // Enable CAN FD2 clock
    CLK_EnableModuleClock(CANFD3_MODULE);                                    // Enable CAN FD3 clock

    /*--------------------------------------------------------------------------------------------*/
    /* Init I/O Multi-function                                                                    */
    /*--------------------------------------------------------------------------------------------*/

    SET_UART0_RXD_PB12();       // Set multi-function pins for UART0 RXD and TXD
    SET_UART0_TXD_PB13();
}

/*--------------------------------------------------------------------------------------------*/
/*  main function                                                                             */
/*--------------------------------------------------------------------------------------------*/
int32_t main(void)
{
    uint8_t Mutex_Err ;

    SYS_UnlockReg();                  // Unlock protected registers
    SYS_Init();                       // Init System, peripheral clock and multi-function I/O

    UART_Open(UART0, 115200);                               //  Init UART0 for printf
    printf("\n\nCPU @ %d Hz\n", SystemCoreClock);

    SYS_ResetModule(CANFD0_RST);
    SYS_ResetModule(CANFD1_RST);
    SYS_ResetModule(CANFD2_RST);
    SYS_ResetModule(CANFD3_RST);

    //== Configure CANFD0 =======================================
    CANFD0->CCCR = CANFD_CCCR_CCE_Msk | CANFD_CCCR_INIT_Msk | CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk;
#ifndef  __M467SJHAE            // If test this project on NuMaker-M467HJ board  
    SET_CAN0_RXD_PJ11();        // Set PJ multi-function pins for CAN FD0 RXD and TXD
    SET_CAN0_TXD_PJ10();
#else                           // If test this project on M467SJHAE board  
    SET_CAN0_RXD_PB10();
    SET_CAN0_TXD_PB11();
#endif
    CANFD_BitRate_Init(CANFD0);
    CANFD_Tx_Init(CANFD0, 32, 0xE0);  // 32-Txbuf,StartAddress=0xE0 Please reference CANFD_TxBuff[32]
    CANFD0_RX_Initial(CANFD0);
//    CANFD0->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;    // CANFD frame
    CANFD0->CCCR = 0 ;                                         // CAN frame,Write is OK after 2 CLKs

    //== Configure CANFD1 =======================================
    CANFD1->CCCR = CANFD_CCCR_CCE_Msk | CANFD_CCCR_INIT_Msk | CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk;
#ifndef  __M467SJHAE            // If test this project on NuMaker-M467HJ board  
    SET_CAN1_RXD_PJ13();
    SET_CAN1_TXD_PJ12();
#else                           // If test this project on M467SJHAE board  
    SET_CAN1_RXD_PB6();
    SET_CAN1_TXD_PB7();
#endif
    CANFD_BitRate_Init(CANFD1);
    CANFD_Tx_Init(CANFD1, 32, 0xE0);  // 32-Txbuf,StartAddress=0xE0 Please reference CANFD_TxBuff[32]
    CANFD1_RX_Initial(CANFD1);
//    CANFD1->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame
    CANFD1->CCCR = 0 ;                                         // CAN frame,Write is OK after 2 CLKs

    //== Configure CANFD2 =======================================
    CANFD2->CCCR = CANFD_CCCR_CCE_Msk | CANFD_CCCR_INIT_Msk | CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk;
#ifndef  __M467SJHAE            // If test this project on NuMaker-M467HJ board  
    SET_CAN2_RXD_PJ9();
    SET_CAN2_TXD_PJ8();
#else                           // If test this project on M467SJHAE board  
    SET_CAN2_RXD_PB8();
    SET_CAN2_TXD_PB9();
#endif
    CANFD_BitRate_Init(CANFD2);
    CANFD_Tx_Init(CANFD2, 32, 0xE0);  // 32-Txbuf,StartAddress=0xE0 Please reference CANFD_TxBuff[32]
    CANFD2_RX_Initial(CANFD2);
//    CANFD2->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame
    CANFD2->CCCR = 0 ;                                        // CAN frame, Write is OK after 2 CLKs

    //== Configure CANFD3 =======================================
    CANFD3->CCCR = CANFD_CCCR_CCE_Msk | CANFD_CCCR_INIT_Msk | CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk;
#ifndef  __M467SJHAE            // If test this project on NuMaker-M467HJ board  
    SET_CAN3_RXD_PH13();
    SET_CAN3_TXD_PH12();
#else                           // If test this project on M467SJHAE board  
    SET_CAN3_RXD_PC6();
    SET_CAN3_TXD_PC7();
#endif
    CANFD_BitRate_Init(CANFD3);
    CANFD_Tx_Init(CANFD3, 32, 0xE0);  // 32-Txbuf,StartAddress=0xE0 Please reference CANFD_TxBuff[32]
    CANFD3_RX_Initial(CANFD3);
//    CANFD3->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame
    CANFD3->CCCR = 0 ;                                      // CAN frame, Write is OK after 2 CLKs

    PH->DOUT |= 0x60 ;                                      // This PIO is indication
    GPIO_SetMode(PH, BIT4 | BIT5 | BIT6, GPIO_MODE_OUTPUT);
    SYS_LockReg();                                          // Lock protected registers

    OSInit();                                               // uC/OS initialization.
    p_MailBox_1 = OSMboxCreate((void *)0);                  // create a mail box
    pQ_print   = OSQCreate(Print_Message, 10);              // create a Q which have 10 messages

    pMutex_CANFD0_Tx = OSMutexCreate(8,  &Mutex_Err);       // First parameter is priority of Mutex
    pMutex_CANFD1_Tx = OSMutexCreate(9,  &Mutex_Err);
    pMutex_CANFD2_Tx = OSMutexCreate(10, &Mutex_Err);
    pMutex_CANFD3_Tx = OSMutexCreate(11, &Mutex_Err);

    pSem_CANFD0_Rx = OSSemCreate(0);
    pSem_CANFD1_Rx = OSSemCreate(0);
    pSem_CANFD2_Rx = OSSemCreate(0);
    pSem_CANFD3_Rx = OSSemCreate(0);

    OSTaskCreateExt((void(*)(void *))task_Highest,         // create the highest priority task
                    (void *)0,
                    (OS_STK *)&task_High[TASK_STK_SIZE - 1],
                    (INT8U)OS_HIGHEST_PRIO,
                    (INT16U)OS_HIGHEST_PRIO,
                    (OS_STK *)&task_High[0],
                    (INT32U)TASK_STK_SIZE,
                    (void *)0,
                    (INT16U)OS_TASK_OPT_STK_CHK | OS_TASK_OPT_STK_CLR | OS_TASK_OPT_SAVE_FP);

    OSTaskCreate((void(*)(void *))task_A,                      // task function
                 (void *)0,                                    // Parameter of task function
                 (OS_STK *)&taskA_stk[TASK_STK_SIZE - 1],      // stack of task
                 (INT8U)TASKA_PRIO);                           // Priority of task

    OSTaskCreate((void(*)(void *))task0,               // create another task
                 (void *)0,
                 (OS_STK *)&task0_stk[TASK_STK_SIZE - 1],
                 (INT8U)TASK0_PRIO);

    OSTaskCreate((void(*)(void *))task1,               // create another task
                 (void *)0,
                 (OS_STK *)&task1_stk[TASK_STK_SIZE - 1],
                 (INT8U)TASK1_PRIO);

    OSTaskCreate((void(*)(void *))task2,               // create another task
                 (void *)0,
                 (OS_STK *)&task2_stk[TASK_STK_SIZE - 1],
                 (INT8U)TASK2_PRIO);

    OSTaskCreate((void(*)(void *))task3,               // create another task
                 (void *)0,
                 (OS_STK *)&task3_stk[TASK_STK_SIZE - 1],
                 (INT8U)TASK3_PRIO);

    OSStart();                                             // Start schedule

    while (1) {} ;          // This sentence would be never executed.
}

// Caution: Tick-timer MUST be actived in the highest task

// When other task has message to print, the message must be sent to this task by OSQPost(pQ_print,Message)

/*-----------------------------------------------------------------------------------------------*/
/*  Code for highest task                                                                        */
/*-----------------------------------------------------------------------------------------------*/
void task_Highest(void *pvParameters)
{
    uint8_t  u8Cnt, err ;
    uint8_t *ptr ;

    SysTick_Config(SystemCoreClock / OS_TICKS_PER_SEC);    // Tick-timer activing

    while (1)
    {
        ptr = OSQPend(pQ_print, 3, &err);              // Waiting 20~30ms for a message to print

        if (ptr != (void *)0)
        {
            while (*ptr != 0)                          // Not the last character of a string.
            {
                while (UART_IS_TX_FULL(UART0)) OSTimeDly(1);   // Tx_FiFo is full,waitting 10ms

                UART0->DAT = *ptr++ ;                          // write a character into Tx_FiFO
            }
        }

        // Only in this task can Message be printed directly
        do
        {
            u8Cnt = OSSemAccept(pSem_CANFD0_Rx);              // Check weather receive a CAN message

            if (u8Cnt == 0) break ;

            CANFD_ReadRxFifoMsg(CANFD0, 1, &g_sRxMsgFrame);                   // Rx FIFO1 Receive

            if (g_sRxMsgFrame.eIdType == eCANFD_SID)
                printf("\r\nCAN0 SID=0x%04X", g_sRxMsgFrame.u32Id);
            else
                printf("\r\nCAN0 XID=0x%04X", g_sRxMsgFrame.u32Id);

            printf("(%02d Bytes): ", g_sRxMsgFrame.u32DLC);

            for (u8Cnt = 0; u8Cnt <  g_sRxMsgFrame.u32DLC; u8Cnt++)
            {
                printf("%02d,", g_sRxMsgFrame.au8Data[u8Cnt]);
            }
        } while (u8Cnt) ;

        do
        {
            u8Cnt = OSSemAccept(pSem_CANFD1_Rx);              // Check weather receive a CAN message

            if (u8Cnt == 0) break ;

            CANFD_ReadRxFifoMsg(CANFD1, 1, &g_sRxMsgFrame);                   // Rx FIFO1 Receive

            if (g_sRxMsgFrame.eIdType == eCANFD_SID)
                printf("\r\nCAN1 SID=0x%04X", g_sRxMsgFrame.u32Id);
            else
                printf("\r\nCAN1 XID=0x%04X", g_sRxMsgFrame.u32Id);

            printf("(%02d Bytes): ", g_sRxMsgFrame.u32DLC);

            for (u8Cnt = 0; u8Cnt <  g_sRxMsgFrame.u32DLC; u8Cnt++)
            {
                printf("%02d,", g_sRxMsgFrame.au8Data[u8Cnt]);
            }
        } while (u8Cnt) ;

        do
        {
            u8Cnt = OSSemAccept(pSem_CANFD2_Rx);              // Check weather receive a CAN message

            if (u8Cnt == 0) break ;

            CANFD_ReadRxFifoMsg(CANFD2, 1, &g_sRxMsgFrame);                   // Rx FIFO1 Receive

            if (g_sRxMsgFrame.eIdType == eCANFD_SID)
                printf("\r\nCAN2 SID=0x%04X", g_sRxMsgFrame.u32Id);
            else
                printf("\r\nCAN2 XID=0x%04X", g_sRxMsgFrame.u32Id);

            printf("(%02d Bytes): ", g_sRxMsgFrame.u32DLC);

            for (u8Cnt = 0; u8Cnt <  g_sRxMsgFrame.u32DLC; u8Cnt++)
            {
                printf("%02d,", g_sRxMsgFrame.au8Data[u8Cnt]);
            }
        } while (u8Cnt) ;

        do
        {
            u8Cnt = OSSemAccept(pSem_CANFD3_Rx);              // Check weather receive a CAN message

            if (u8Cnt == 0) break ;

            CANFD_ReadRxFifoMsg(CANFD3, 1, &g_sRxMsgFrame);                   // Rx FIFO1 Receive

            if (g_sRxMsgFrame.eIdType == eCANFD_SID)
                printf("\r\nCAN3 SID=0x%04X", g_sRxMsgFrame.u32Id);
            else
                printf("\r\nCAN3 XID=0x%04X", g_sRxMsgFrame.u32Id);

            printf("(%02d Bytes): ", g_sRxMsgFrame.u32DLC);

            for (u8Cnt = 0; u8Cnt <  g_sRxMsgFrame.u32DLC; u8Cnt++)
            {
                printf("%02d,", g_sRxMsgFrame.au8Data[u8Cnt]);
            }
        } while (u8Cnt) ;
    }
}

/*-----------------------------------------------------------------------------------------------*/
/*  Task A code                                                                                  */
/*-----------------------------------------------------------------------------------------------*/
void task_A(void *pvParameters)
{
    uint8_t  Mutex_Err ;

    while (1)
    {
        OSTimeDlyHMSM(0, 0, 12, 0);                     // Delay 12 seconds

        if (Mail_Message[0] != 6000) Mail_Message[0] = 6000;
        else Mail_Message[0] = 3000 ;

        OSMboxPost(p_MailBox_1, Mail_Message);          // Send message(time) to task2 via mail box

        OSMutexPend(pMutex_CANFD0_Tx, 0, &Mutex_Err);   // Pend by CANFD0 Mutex
        CANFD0_Buffer_2_Transmission() ;                // CANFD2 transmission
        Mutex_Err = OSMutexPost(pMutex_CANFD0_Tx) ;     // Post CANFD0 Mutex

        OSQPost(pQ_print, "\r\n\r\n  Task A\r\n");      // Send print message to task3 by Q
        LED_G_TOGGLE() ;                                // Toggle LED to indicate
    }
}

/*-----------------------------------------------------------------------------------------------*/
/*  Task 0 code                                                                                  */
/*-----------------------------------------------------------------------------------------------*/
void task0(void *pvParameters)
{
    int32_t  *ptr ;
    int16_t  static Second = 3, mSecond = 0 ;
    //    uint8_t  Mutex_Err ;

    while (1)
    {
        ptr = OSMboxAccept(p_MailBox_1);          // Check whether there are message in mail box

        if (ptr != (void *)0)                     // Received a mail message
        {
            mSecond = *ptr ;                               // Read out the message in mailbox
            Second   = mSecond / 1000 ;
            mSecond -= Second * 1000 ;                     // Update Time
        }

        // If Bus-Off occurs, CCCR[0] must be written in order to cancel bus-Off.

        // CANFD0->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame,Write is OK after 2 CLKs
        CANFD0->CCCR = 0 ;                                            // CAN frame
        OSTimeDlyHMSM(0, 0, Second, mSecond);              // Delay time according to message.

        // If some of 32 TxBuffer are written ONLY in this task, MuteX needn't be defined and used.

        // OSMutexPend(pMutex_CANFD0_Tx, 0, &Mutex_Err);   // Pend by CANFD0 Mutex
        CANFD0_Buffer_0_Transmission() ;                   // CANFD0 transmission
        // Mutex_Err = OSMutexPost(pMutex_CANFD0_Tx) ;     // Post CANFD0 Mutex

        OSQPost(pQ_print, "\r\nTask 0");      // Send print-message to task3
        LED_Y_TOGGLE() ;                                // LED_Y toggle(flash)
    }
}

/*-----------------------------------------------------------------------------------------------*/
/*  Task 1 code                                                                                  */
/*-----------------------------------------------------------------------------------------------*/
void task1(void *pvParameters)
{
    uint8_t  Mutex_Err ;

    while (1)
    {
        // If Bus-Off occurs, CCCR[0] must be written in order to cancel bus-Off.

        // CANFD1->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame,Write is OK after 2 CLKs
        CANFD1->CCCR = 0 ;                                            // CAN frame
        OSTimeDlyHMSM(0, 0, 3, 0);                      // Delay time according to message.

        OSMutexPend(pMutex_CANFD1_Tx, 0, &Mutex_Err);   // Pend by CANFD0 Mutex
        CANFD1_Buffer_0_Transmission() ;                // CANFD0 transmission
        Mutex_Err = OSMutexPost(pMutex_CANFD1_Tx) ;     // Post CANFD0 Mutex

        OSQPost(pQ_print, "\r\nTask 1");                // Send print-message to task3
        LED_Y_TOGGLE() ;                                // LED_Y toggle(flash)
    }
}

/*-----------------------------------------------------------------------------------------------*/
/*  Task 2 code                                                                                  */
/*-----------------------------------------------------------------------------------------------*/
void task2(void *pvParameters)
{
    uint8_t  Mutex_Err ;

    while (1)
    {
        // If Bus-Off occurs, CCCR[0] must be written in order to cancel bus-Off.

        // CANFD2->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame,Write is OK after 2 CLKs
        CANFD2->CCCR = 0 ;                                          // CAN frame
        OSTimeDlyHMSM(0, 0, 3, 0);                      // Delay time according to message.

        OSMutexPend(pMutex_CANFD2_Tx, 0, &Mutex_Err);   // Pend by CANFD0 Mutex
        CANFD2_Buffer_0_Transmission() ;                // CANFD0 transmission
        Mutex_Err = OSMutexPost(pMutex_CANFD2_Tx) ;     // Post CANFD0 Mutex

        OSQPost(pQ_print, "\r\nTask 2");                // Send print-message to task3
        LED_Y_TOGGLE() ;                                // LED_Y toggle(flash)
    }
}

/*-----------------------------------------------------------------------------------------------*/
/*  Task 3 code                                                                                  */
/*-----------------------------------------------------------------------------------------------*/
void task3(void *pvParameters)
{
    uint8_t  Mutex_Err ;

    while (1)
    {
        // If Bus-Off occurs, CCCR[0] must be written in order to cancel bus-Off.

        // CANFD3->CCCR = CANFD_CCCR_BRSE_Msk | CANFD_CCCR_FDOE_Msk ;  //CANFD frame,Write is OK after 2 CLKs
        CANFD3->CCCR = 0 ;                                          // CAN frame
        OSTimeDlyHMSM(0, 0, 3, 0);                      // Delay time according to message.

        OSMutexPend(pMutex_CANFD3_Tx, 0, &Mutex_Err);   // Pend by CANFD0 Mutex
        CANFD3_Buffer_0_Transmission() ;                // CANFD0 transmission
        Mutex_Err = OSMutexPost(pMutex_CANFD3_Tx) ;     // Post CANFD0 Mutex

        OSQPost(pQ_print, "\r\nTask 3");                // Send print-message to task3
        LED_Y_TOGGLE() ;                                // LED_Y toggle(flash)
    }
}

/*-----------------------------------------------------------------------------------------------*/
/*  ISR of CANFD 0 interrupt                                                              */
/*-----------------------------------------------------------------------------------------------*/
void CANFD00_IRQHandler(void)
{
#if OS_CRITICAL_METHOD == 3u                      // Allocate storage for CPU status register   
    OS_CPU_SR  cpu_sr;
#endif

    OS_ENTER_CRITICAL();
    OSIntEnter();                                 // Tell uC/OS-II that we are starting an ISR
    OS_EXIT_CRITICAL();

    OSSemPost(pSem_CANFD0_Rx);                                            // post a semaphone
    CANFD_ClearStatusFlag(CANFD0, CANFD_IR_TOO_Msk | CANFD_IR_RF1N_Msk);  // Clear Interrupt flag

    OSIntExit();                                  // Tell uC/OS-II that we are leaving the ISR
}

/*-----------------------------------------------------------------------------------------------*/
/*  ISR of CANFD 1 interrupt                                                              */
/*-----------------------------------------------------------------------------------------------*/
void CANFD10_IRQHandler(void)
{
#if OS_CRITICAL_METHOD == 3u                      // Allocate storage for CPU status register   
    OS_CPU_SR  cpu_sr;
#endif

    OS_ENTER_CRITICAL();
    OSIntEnter();                                 // Tell uC/OS-II that we are starting an ISR
    OS_EXIT_CRITICAL();

    OSSemPost(pSem_CANFD1_Rx);                                            // post a semaphone
    CANFD_ClearStatusFlag(CANFD1, CANFD_IR_TOO_Msk | CANFD_IR_RF1N_Msk);  // Clear Interrupt flag

    OSIntExit();                                  // Tell uC/OS-II that we are leaving the ISR
}

/*-----------------------------------------------------------------------------------------------*/
/*  ISR of CANFD 2 interrupt                                                              */
/*-----------------------------------------------------------------------------------------------*/
void CANFD20_IRQHandler(void)
{
#if OS_CRITICAL_METHOD == 3u                      // Allocate storage for CPU status register   
    OS_CPU_SR  cpu_sr;
#endif

    OS_ENTER_CRITICAL();
    OSIntEnter();                                 // Tell uC/OS-II that we are starting an ISR
    OS_EXIT_CRITICAL();

    OSSemPost(pSem_CANFD2_Rx);                                            // post a semaphone
    CANFD_ClearStatusFlag(CANFD2, CANFD_IR_TOO_Msk | CANFD_IR_RF1N_Msk);  // Clear Interrupt flag

    OSIntExit();                                  // Tell uC/OS-II that we are leaving the ISR
}

/*-----------------------------------------------------------------------------------------------*/
/*  ISR of CANFD 3 interrupt                                                              */
/*-----------------------------------------------------------------------------------------------*/
void CANFD30_IRQHandler(void)
{
#if OS_CRITICAL_METHOD == 3u                      // Allocate storage for CPU status register   
    OS_CPU_SR  cpu_sr;
#endif

    OS_ENTER_CRITICAL();
    OSIntEnter();                                 // Tell uC/OS-II that we are starting an ISR
    OS_EXIT_CRITICAL();

    OSSemPost(pSem_CANFD3_Rx);                                            // post a semaphone
    CANFD_ClearStatusFlag(CANFD3, CANFD_IR_TOO_Msk | CANFD_IR_RF1N_Msk);  // Clear Interrupt flag

    OSIntExit();                                  // Tell uC/OS-II that we are leaving the ISR
}
